require "json"
require "net/ldap"
require "seiso/connector"
require_relative "./import_ldap/authentication_failed_error"

module Seiso

  # Imports LDAP person data into Seiso.
  #
  # Author:: Willie Wheeler (mailto:wwheeler@expedia.com)
  # Copyright:: Copyright (c) 2014-2015 Expedia, Inc.
  # License:: Apache 2.0
  class ImportLdap
    BATCH_SIZE = 20
    
    # OpenLDAP: http://oav.net/mirrors/LDAP-ObjectClasses.html
  
    # Active Directory: http://msdn.microsoft.com/en-us/library/ms675090(v=vs.85).aspx
    ACTIVE_DIRECTORY_ATTRS = [
      "dn",
      "sAMAccountName",
      "name",
      "givenName",
      "sn",
      "title",
      "company",
      "department",
      "division",
      "l",
      "streetAddress",
      "telephoneNumber",
      "mail",
      "manager"
      
      # Not currently using these
      # "objectClass",
      # "objectCategory",
      # "cn",
      # "displayName",
      # "userPrincipalName",
      # "st",
      # "postalCode",
      # "co",
      # "c",
      # "physicalDeliveryOfficeName",
      # "showInAddressBook",
      # "memberOf",
      # "managedObjects",
      # "whenCreated",
      # "whenChanged",
      # "logonCount",
      # "lastLogonTimestamp"
    ]
    
    # Creates a new LDAP importer.
    # - ldap_settings: Has the following settings
    #   - host
    #   - port
    #   - username
    #   - password
    #   - ldap_base_dn: tree base (for searching)
    #   - ldap_user_dn: user filter (for searching)
    # - seiso_settings
    # See sample-conf for example configurations.
    def initialize(ldap_settings, seiso_settings)
      @source = ldap_settings['source']

      # LDAP connector
      @ldap = Net::LDAP.new
      @ldap.host = ldap_settings['host']
      @ldap.port = ldap_settings['port']
      @ldap.auth ldap_settings['username'], ldap_settings['password']

      # Seiso connector
      @seiso = Seiso::Connector.new seiso_settings

      # LDAP search params
      @tree_base = ldap_settings['tree_base']
      @person_filter = Net::LDAP::Filter::FilterParser.parse(ldap_settings['person_filter'])
      puts "tree_base=#{@tree_base}"
      puts "person_filter=#{@person_filter}"
    end

    def ldap
      @ldap
    end

    def seiso
      @seiso
    end
    
    def import_all
      if !ldap.bind
        raise Seiso::ImportLdap::AuthenticationFailedError
      end
      
      # TODO Currently we assume Active Directory. Will generalize if/when somebody asks for it.
      attrs = ACTIVE_DIRECTORY_ATTRS

      # Hash so we can resolve managers by distinguished name (DN)
      users = {}
      ldap.search(:base => @tree_base, :filter => @person_filter, :attributes => attrs) do |user|
        dn = single_value(user, "dn")
        users[dn] = user
      end

      # Step #1: Delete stale users
#      seiso_users = seiso.get_keys("people", @source, 0)
#      puts seiso_users
      # TODO
      
      # Step #2: Create users
      people = []
      users.each do |dn, user|
        people << to_seiso_person(user)
      end

      # TODO Page this, and use PATCH instead of PUT to avoid nulling out managers
      seiso.post_items("people", people)
      
      # Step #3: Link managers
      # TODO
    end

    private

    def to_seiso_person(user)
      {
        "username" => single_value(user, "sAMAccountName"),
        "firstName" => single_value(user, "givenName"),
        "lastName" => single_value(user, "sn"),
        "title" => single_value(user, "title"),
        "company" => single_value(user, "company"),
        "department" => single_value(user, "department"),
        "division" => single_value(user, "division"),
        "location" => single_value(user, "l"),
        "streetAddress" => single_value(user, "streetAddress"),
        "workPhone" => single_value(user, "telephoneNumber"),
        "email" => single_value(user, "mail"),
        "ldapDn" => single_value(user, "dn"),
        "source" => @source
      }
    end

    def single_value(user, attr)
      values = user[attr]
      (values.nil? || values.empty?) ? nil : values[0].to_s.force_encoding("UTF-8")
    end
  end
end
